package com.example.demo;

import java.util.List;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.MethodOrderer.OrderAnnotation;
import org.junit.jupiter.api.Order;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.dao.DataIntegrityViolationException;

import com.example.demo.authors.model.AuthorEntity;
import com.example.demo.authors.service.AuthorService;
import com.example.demo.books.model.BookEntity;
import com.example.demo.books.service.BookService;
import com.example.demo.core.error.NotFoundException;
import com.example.demo.genres.model.GenreEntity;
import com.example.demo.genres.service.GenreService;

@SpringBootTest
@TestMethodOrder(OrderAnnotation.class)
class BookServiceTests {
    @Autowired
    private BookService bookService;

    private BookEntity book;
    private BookEntity book2;
    private BookEntity book3;
    private BookEntity book4;

    @Autowired
    private GenreService genreService;

    private GenreEntity genre;
    private GenreEntity genre2;

    @Autowired
    private AuthorService authorService;

    private AuthorEntity author;
    private AuthorEntity author2;

    @BeforeEach
    void createData() {
        removeData();

        genre = genreService.create(new GenreEntity("Мемуары"));
        genre2 = genreService.create(new GenreEntity("Фантастика"));

        author = authorService.findByName("Вася Пупкин");
        if (author == null) {
            author = authorService.create(new AuthorEntity("Вася Пупкин"));
        }

        author2 = authorService.findByName("Петя Иванов");
        if (author2 == null) {
            author2 = authorService.create(new AuthorEntity("Петя Иванов"));
        }

        book = bookService.create(new BookEntity(genre, author, "Книга 1", "1", "Описание 1"));
        book2 = bookService.create(new BookEntity(genre, author, "Книга 2", "2", "Описание 2"));
        book3 = bookService.create(new BookEntity(genre2, author, "Книга 3", "3", "Описание 3"));
        book4 = bookService.create(new BookEntity(genre, author2, "Книга 4", "4", "Описание 4"));
    }

    @AfterEach
    void removeData() {
        bookService.getAll(0, 0).forEach(item -> bookService.delete(item.getId()));
        genreService.getAll().forEach(item -> genreService.delete(item.getId()));
    }

    @Test
    @Order(1)
    void getTest() {
        Assertions.assertThrows(NotFoundException.class, () -> bookService.get(10L));
    }

    @Test
    @Order(2)
    void createTest() {
        Assertions.assertEquals(2, bookService.getAll(genre.getId(), author.getId()).size());
        Assertions.assertEquals(book, bookService.get(book.getId()));
    }

    @Test
    @Order(3)
    void createNullableTest() {
        final BookEntity nullableGenre = new BookEntity(genre, null, "1", "1", "1");
        Assertions.assertThrows(DataIntegrityViolationException.class, () -> bookService.create(
                nullableGenre));
    }

    @Test
    @Order(4)
    void updateTest() {
        final String newName = "TEST";
        final String oldName = book.getName();
        final BookEntity mov = new BookEntity(book.getGenre(), book.getAuthor(),
                newName, book.getYear(), book.getDescription());
        final BookEntity newEntity = bookService.update(book.getId(), mov);
        Assertions.assertEquals(newName, newEntity.getName());
        Assertions.assertNotEquals(oldName, newEntity.getName());
    }

    @Test
    @Order(5)
    void deleteTest() {
        bookService.delete(book2.getId());
        Assertions.assertEquals(1, bookService.getAll(genre.getId(),
                author.getId()).size());

        Assertions.assertNotEquals(3, bookService.getAll(genre.getId(),
                author.getId()).size());
    }

    @Test
    @Order(6)
    void filterByGenreTest() {
        List<BookEntity> books = bookService.getAll(genre.getId(), 0);
        Assertions.assertFalse(books.isEmpty());
        Assertions.assertTrue(books.stream().allMatch(b -> b.getGenre().equals(genre)));
        Assertions.assertEquals(3, books.size());
    }

    @Test
    @Order(7)
    void filterByAuthorTest() {
        List<BookEntity> books = bookService.getAll(0, author2.getId());
        Assertions.assertFalse(books.isEmpty());
        Assertions.assertTrue(books.stream().allMatch(b -> b.getAuthor().equals(author2)));
        Assertions.assertEquals(1, books.size());
    }

    @Test
    @Order(8)
    void filterByGenreAndAuthorTest() {
        List<BookEntity> books = bookService.getAll(genre.getId(), author.getId());
        Assertions.assertFalse(books.isEmpty());
        Assertions.assertTrue(books.stream().allMatch(b -> b.getGenre().equals(genre) && b.getAuthor().equals(author)));
        Assertions.assertEquals(2, books.size());
    }
}
