package com.example.demo;

import java.util.List;
import java.util.Objects;
import java.util.stream.IntStream;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.CommandLineRunner;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;

import com.example.demo.authors.model.AuthorEntity;
import com.example.demo.authors.service.AuthorService;
import com.example.demo.books.model.BookEntity;
import com.example.demo.books.service.BookService;
import com.example.demo.core.configuration.Constants;
import com.example.demo.favorites.model.FavoriteEntity;
import com.example.demo.favorites.service.FavoriteService;
import com.example.demo.genres.model.GenreEntity;
import com.example.demo.genres.service.GenreService;
import com.example.demo.users.model.UserEntity;
import com.example.demo.users.model.UserRole;
import com.example.demo.users.service.UserService;

@SpringBootApplication
public class DemoApplication implements CommandLineRunner {
    private final Logger log = LoggerFactory.getLogger(DemoApplication.class);

    private final GenreService genreService;
    private final AuthorService authorService;
    private final BookService bookService;
    private final UserService userService;
    private final FavoriteService favoriteService;

    public DemoApplication(
            UserService userService,
            GenreService genreService,
            AuthorService authorService,
            BookService bookService,
            FavoriteService favoriteService) {
        this.userService = userService;
        this.genreService = genreService;
        this.authorService = authorService;
        this.bookService = bookService;
        this.favoriteService = favoriteService;
    }

    public static void main(String[] args) {
        SpringApplication.run(DemoApplication.class, args);
    }

    @Override
    public void run(String... args) throws Exception {
        if (args.length > 0 && Objects.equals("--populate", args[0])) {
            log.info("Create default genre values");
            final var genre1 = genreService.create(new GenreEntity("Детектив"));
            final var genre2 = genreService.create(new GenreEntity("Роман"));
            final var genre3 = genreService.create(new GenreEntity("Трагедия"));

            log.info("Create default author values");
            final var author1 = authorService.create(new AuthorEntity("А.С.Пушкин"));
            final var author2 = authorService.create(new AuthorEntity("М.Ю.Леромонтов"));
            final var author3 = authorService.create(new AuthorEntity("Л.Н.Толстой"));

            log.info("Create default book values");
            final var book1 = bookService.create(new BookEntity(
                    genre1, author1, "Книга1", "1900", "описание1"));
            final var book2 = bookService.create(new BookEntity(
                    genre2, author2, "Книга2", "1890", "описание2"));
            final var book3 = bookService.create(new BookEntity(
                    genre3, author3, "Книга3", "1890", "описание3"));

            log.info("Create default user values");
            final var admin = new UserEntity("admin", "admin");
            admin.setRole(UserRole.ADMIN);
            userService.create(admin);

            final var user1 = userService.create(new UserEntity("user1", Constants.DEFAULT_PASSWORD));

            IntStream.range(2, 27)
                    .forEach(value -> userService.create(
                            new UserEntity("user".concat(String.valueOf(value)), Constants.DEFAULT_PASSWORD)));

            log.info("Create default favorite values");
            final var favorites = List.of(
                    new FavoriteEntity(user1, book1),
                    new FavoriteEntity(user1, book2),
                    new FavoriteEntity(user1, book3));
            favorites.forEach(favorite -> favoriteService.create(favorite, user1.getId()));
        }
    }
}
