package com.example.demo.books.api;

import java.util.Map;

import org.modelmapper.ModelMapper;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.example.demo.core.api.PageAttributesMapper;
import com.example.demo.core.configuration.Constants;
import com.example.demo.authors.api.AuthorDto;
import com.example.demo.authors.model.AuthorEntity;
import com.example.demo.authors.service.AuthorService;
import com.example.demo.books.model.BookEntity;
import com.example.demo.books.service.BookService;
import com.example.demo.genres.api.GenreDto;
import com.example.demo.genres.model.GenreEntity;
import com.example.demo.genres.service.GenreService;

import jakarta.validation.Valid;

@Controller
@RequestMapping(BookController.URL)
public class BookController {
    public static final String URL = Constants.ADMIN_PREFIX + "/book";
    private static final String BOOK_VIEW = "book";
    private static final String BOOK_EDIT_VIEW = "book-edit";
    private static final String BOOK_ATTRIBUTE = "book";

    private static final String PAGE_ATTRIBUTE = "page";

    private final BookService bookService;
    private final GenreService genreService;
    private final AuthorService authorService;
    private final ModelMapper modelMapper;

    public BookController(BookService bookService, GenreService genreService, AuthorService authorService,
            ModelMapper modelMapper) {
        this.bookService = bookService;
        this.modelMapper = modelMapper;
        this.genreService = genreService;
        this.authorService = authorService;
    }

    private BookDto toDto(BookEntity entity) {
        return modelMapper.map(entity, BookDto.class);
    }

    private BookEntity toEntity(BookDto dto) {
        final BookEntity entity = modelMapper.map(dto, BookEntity.class);
        entity.setGenre(genreService.get(dto.getGenreId()));
        entity.setAuthor(authorService.get(dto.getAuthorId()));
        return entity;
    }

    private GenreDto toGenreDto(GenreEntity entity) {
        return modelMapper.map(entity, GenreDto.class);
    }

    private AuthorDto toAuthorDto(AuthorEntity entity) {
        return modelMapper.map(entity, AuthorDto.class);
    }

    public String getAllBooks(@RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            Model model) {

        final Map<String, Object> attributes = PageAttributesMapper.toAttributes(
                bookService.getAllBooks(page, Constants.DEFUALT_PAGE_SIZE), this::toDto);

        model.addAllAttributes(attributes);
        model.addAttribute(PAGE_ATTRIBUTE, page);

        return BOOK_VIEW;
    }

    @GetMapping
    public String getAll(@RequestParam(name = "genreId", defaultValue = "0") Long genreId,
            @RequestParam(name = "authorId", defaultValue = "0") Long authorId,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            Model model) {
        final Map<String, Object> attributes = PageAttributesMapper.toAttributes(
                bookService.getAllBooks(page, Constants.DEFUALT_PAGE_SIZE), this::toDto);

        // book.setGenreName(genreService.get(book.getGenreId()));

        model.addAllAttributes(attributes);
        model.addAttribute(PAGE_ATTRIBUTE, page);

        model.addAttribute(
                "genres",
                genreService.getAll().stream().map(this::toGenreDto).toList());

        model.addAttribute(
                "authors",
                authorService.getAll().stream().map(this::toAuthorDto).toList());

        model.addAttribute(
                "books",
                bookService.getAllBooks(page, Constants.DEFUALT_PAGE_SIZE).stream().map(this::toDto).toList());

        return BOOK_VIEW;
    }

    @GetMapping("/edit/")
    public String create(@RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page, Model model) {
        model.addAttribute(BOOK_ATTRIBUTE, new BookDto());
        model.addAttribute("genres", genreService.getAll().stream().map(this::toGenreDto).toList());
        model.addAttribute("authors", authorService.getAll().stream().map(this::toAuthorDto).toList());
        model.addAttribute(PAGE_ATTRIBUTE, page);
        return BOOK_EDIT_VIEW;
    }

    @PostMapping("/edit/")
    public String create(@RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            @ModelAttribute(name = BOOK_ATTRIBUTE) @Valid BookDto book,
            BindingResult bindingResult, Model model, RedirectAttributes redirectAttributes) {

        if (bindingResult.hasErrors()) {
            model.addAttribute(PAGE_ATTRIBUTE, page);
            return BOOK_EDIT_VIEW;
        }

        redirectAttributes.addAttribute(PAGE_ATTRIBUTE, page);
        bookService.create(toEntity(book));
        return Constants.REDIRECT_VIEW + URL;
    }

    @GetMapping("/edit/{id}")
    public String update(@PathVariable(name = "id") Long id,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            Model model) {

        model.addAttribute(BOOK_ATTRIBUTE, toDto(bookService.get(id)));

        model.addAttribute("books", bookService.getAllBooks().stream().map(this::toDto).toList());
        model.addAttribute("genres", genreService.getAll().stream().map(this::toGenreDto).toList());
        model.addAttribute("authors", authorService.getAll().stream().map(this::toAuthorDto).toList());
        model.addAttribute(PAGE_ATTRIBUTE, page);
        return BOOK_EDIT_VIEW;
    }

    @PostMapping("/edit/{id}")
    public String update(@PathVariable(name = "id") Long id,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            @ModelAttribute(name = BOOK_ATTRIBUTE) @Valid BookDto book,
            BindingResult bindingResult, Model model, RedirectAttributes redirectAttributes) {

        if (bindingResult.hasErrors()) {
            model.addAttribute(PAGE_ATTRIBUTE, page);
            return BOOK_EDIT_VIEW;
        }

        if (id <= 0) {
            throw new IllegalArgumentException();
        }

        model.addAttribute(
                "genres",
                genreService.getAll().stream().map(this::toGenreDto).toList());
        model.addAttribute(
                "authors",
                authorService.getAll().stream().map(this::toAuthorDto).toList());

        redirectAttributes.addAttribute(PAGE_ATTRIBUTE, page);
        bookService.update(id, toEntity(book));
        return Constants.REDIRECT_VIEW + URL;
    }

    @PostMapping("/delete/{id}")
    public String delete(@PathVariable(name = "id") Long id,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            RedirectAttributes redirectAttributes) {
        redirectAttributes.addAttribute(PAGE_ATTRIBUTE, page);
        bookService.delete(id);

        return Constants.REDIRECT_VIEW + URL;
    }
}
