package com.example.demo.books.api;

import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

import org.modelmapper.ModelMapper;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;

import com.example.demo.authors.api.AuthorDto;
import com.example.demo.authors.model.AuthorEntity;
import com.example.demo.authors.service.AuthorService;
import com.example.demo.books.model.BookEntity;
import com.example.demo.books.service.BookService;
import com.example.demo.core.api.PageAttributesMapper;
import com.example.demo.core.configuration.Constants;
import com.example.demo.genres.api.GenreDto;
import com.example.demo.genres.model.GenreEntity;
import com.example.demo.genres.service.GenreService;

@Controller
@RequestMapping(CatalogController.URL)
public class CatalogController {
    private static final String CATALOG_VIEW = "catalog";
    private static final String URL = "/catalog";

    private static final String BOOK_ATTRIBUTE = "book";
    private static final String BOOK_ONE = "book-one";

    private static final String BOOK_RANDOM = "random-books";

    private static final String GENREID_ATTRIBUTE = "genreId";
    private static final String AUTHORID_ATTRIBUTE = "authorId";

    private static final String PAGE_ATTRIBUTE = "page";

    private final BookService bookService;
    private final GenreService genreService;
    private final AuthorService authorService;
    private final ModelMapper modelMapper;

    public CatalogController(BookService bookService, GenreService genreService,
            AuthorService authorService, ModelMapper modelMapper) {
        this.bookService = bookService;
        this.modelMapper = modelMapper;
        this.genreService = genreService;
        this.authorService = authorService;
    }

    private BookDto toDto(BookEntity entity) {
        return modelMapper.map(entity, BookDto.class);
    }

    private GenreDto toGenreDto(GenreEntity entity) {
        return modelMapper.map(entity, GenreDto.class);
    }

    private AuthorDto toAuthorDto(AuthorEntity entity) {
        return modelMapper.map(entity, AuthorDto.class);
    }

    @GetMapping
    public String getCatalog(@RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            @RequestParam(name = GENREID_ATTRIBUTE, defaultValue = "0") long genreId,
            @RequestParam(name = AUTHORID_ATTRIBUTE, defaultValue = "0") long authorId,
            Model model) {

        model.addAttribute(PAGE_ATTRIBUTE, page);

        model.addAllAttributes(PageAttributesMapper.toAttributes(
                bookService.getAll(genreId, authorId, page, Constants.DEFUALT_PAGE_SIZE), this::toDto));

        model.addAttribute(
                "genres",
                genreService.getAll().stream().map(this::toGenreDto).toList());
        model.addAttribute("genreId", genreId);

        model.addAttribute(
                "authors",
                authorService.getAll().stream().map(this::toAuthorDto).toList());
        model.addAttribute("authorId", authorId);

        return CATALOG_VIEW;
    }

    @GetMapping("/book-one/{id}")
    public String getOne(@PathVariable(name = "id") Long id,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            Model model) {

        model.addAttribute(BOOK_ATTRIBUTE, toDto(bookService.get(id)));

        model.addAttribute("books", bookService.getAllBooks().stream().map(this::toDto).toList());
        model.addAttribute("genres", genreService.getAll().stream().map(this::toGenreDto).toList());
        model.addAttribute("authors", authorService.getAll().stream().map(this::toAuthorDto).toList());
        model.addAttribute(PAGE_ATTRIBUTE, page);
        return BOOK_ONE;
    }

    @GetMapping("/random-books")
    public String getRandomBooks(@RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page, Model model) {
        List<BookDto> allBooks = bookService.getAllBooks().stream().map(this::toDto).collect(Collectors.toList());
        Collections.shuffle(allBooks);
        List<BookDto> randomBooks = allBooks.stream().limit(3).collect(Collectors.toList());

        model.addAttribute("randomBooks", randomBooks);
        model.addAttribute("genres", genreService.getAll().stream().map(this::toGenreDto).toList());
        model.addAttribute("authors", authorService.getAll().stream().map(this::toAuthorDto).toList());
        model.addAttribute(PAGE_ATTRIBUTE, page);

        return BOOK_RANDOM;
    }
}
