package com.example.demo.favorites.api;

import java.util.List;
import java.util.Map;
import java.util.HashMap;
import java.util.stream.Collectors;

import org.modelmapper.ModelMapper;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.example.demo.core.api.PageAttributesMapper;
import com.example.demo.core.configuration.Constants;
import com.example.demo.core.security.UserPrincipal;
import com.example.demo.favorites.model.FavoriteEntity;
import com.example.demo.favorites.service.FavoriteService;
import com.example.demo.genres.api.GenreDto;
import com.example.demo.genres.model.GenreEntity;
import com.example.demo.genres.service.GenreService;
import com.example.demo.authors.api.AuthorDto;
import com.example.demo.authors.model.AuthorEntity;
import com.example.demo.authors.service.AuthorService;
import com.example.demo.books.api.BookDto;
import com.example.demo.books.model.BookEntity;
import com.example.demo.books.service.BookService;
import com.example.demo.users.api.UserDto;
import com.example.demo.users.model.UserEntity;
import com.example.demo.users.service.UserService;

import jakarta.validation.Valid;

@Controller
@RequestMapping(FavoriteController.URL)
public class FavoriteController {

    public static final String URL = "/favorite";
    private static final String FAVORITE_VIEW = "favorite";
    private static final String FAVORITE_EDIT_VIEW = "favorite-edit";
    private static final String FAVORITE_ATTRIBUTE = "favorite";
    private static final String FAVORITE_DETAILS_VIEW = "favorite-details";

    protected static final String PAGE_ATTRIBUTE = "page";
    protected String REDIRECT_AFTER_FAVOURITE_CLICK_PATH;
    protected static final String GENREID_ATTRIBUTE = "genreId";
    protected static final String AUTHORID_ATTRIBUTE = "authorId";

    private final FavoriteService favoriteService;
    private final UserService userService;
    private final BookService bookService;
    private final AuthorService authorService;
    private final GenreService genreService;
    private final ModelMapper modelMapper;

    public FavoriteController(FavoriteService favoriteService, UserService userService,
            BookService bookService, AuthorService authorService, GenreService genreService, ModelMapper modelMapper) {
        this.modelMapper = modelMapper;
        this.userService = userService;
        this.favoriteService = favoriteService;
        this.bookService = bookService;
        this.authorService = authorService;
        this.genreService = genreService;
    }

    private FavoriteDto toDto(FavoriteEntity entity) {
        return modelMapper.map(entity, FavoriteDto.class);
    }

    private FavoriteEntity toEntity(FavoriteDto dto) {
        final FavoriteEntity entity = modelMapper.map(dto, FavoriteEntity.class);
        entity.setBook(bookService.get(dto.getBookId()));
        // entity.setUser(userService.get(dto.getUserId()));
        return entity;
    }

    private UserDto toUserDto(UserEntity entity) {
        return modelMapper.map(entity, UserDto.class);
    }

    private BookDto toBookDto(BookEntity entity, Long userId) {
        BookDto dto = modelMapper.map(entity, BookDto.class);
        dto.setIsFavourite(true);
        dto.setGenreId(entity.getGenre().getId());
        return dto;
    }

    private GenreDto toGenreDto(GenreEntity entity) {
        return modelMapper.map(entity, GenreDto.class);
    }

    private AuthorDto toAuthorDto(AuthorEntity entity) {
        return modelMapper.map(entity, AuthorDto.class);
    }

    @GetMapping
    public String getAll(
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            @RequestParam(name = GENREID_ATTRIBUTE, defaultValue = "0") Long genreId,
            @RequestParam(name = AUTHORID_ATTRIBUTE, defaultValue = "0") Long authorId,
            Model model,
            @AuthenticationPrincipal UserPrincipal principal) {
        final Map<String, Object> attributes = PageAttributesMapper.toAttributes(
                favoriteService.getAll(principal.getId(), page,
                        Constants.DEFUALT_PAGE_SIZE),
                book -> toBookDto(book.getBook(), principal.getId()));
        model.addAllAttributes(attributes);
        model.addAttribute(PAGE_ATTRIBUTE, page);

        model.addAttribute("genres",
                genreService.getAll().stream()
                        .map(this::toGenreDto)
                        .toList());

        model.addAttribute(GENREID_ATTRIBUTE, genreId);

        model.addAttribute("authors",
                authorService.getAll().stream()
                        .map(this::toAuthorDto)
                        .toList());

        model.addAttribute(GENREID_ATTRIBUTE, authorId);

        return FAVORITE_VIEW;
    }

    @PostMapping
    public String toggleFavourite(
            Model model,
            @RequestParam(name = "id") Long bookId,
            @RequestParam(name = "isFavourite") boolean isFavourite,
            @AuthenticationPrincipal UserPrincipal principal) {
        if (bookId <= 0) {
            throw new IllegalArgumentException();
        }

        favoriteService.toggle(principal.getId(), bookId, isFavourite);
        return Constants.REDIRECT_VIEW + REDIRECT_AFTER_FAVOURITE_CLICK_PATH;
    }
}
